unit VarLib;

interface
uses
  classes,
  adoInt;

function MemoryStreamToOlevariant(Strm : TMemoryStream) : Olevariant;
function OleVariantToMemoryStream(OV : OleVariant) : TMemoryStream;
function RecordsetToMemoryStream(const ARecordset : _recordset) : TMemoryStream;
function MemoryStreamToRecordset(const AMemoryStream : TMemoryStream) : _recordset;
{These functions allow recordsets to be passed
even over other protocols}
function BundleRecordset( const ARecordset : _recordset ) : olevariant;
function UnbundleRecordset(const OV : Olevariant) : _recordset;

implementation
uses
  windows, sysutils, activex;

function MemoryStreamToOlevariant(Strm : TMemoryStream) : Olevariant;
var
  Data : PByteArray;
begin
  result := VarArrayCreate([0, Strm.Size -1], varByte);
  Data := VarArrayLock(result);
  try
    Strm.Position := 0;
    Strm.ReadBuffer(Data,Strm.Size);
  finally
    VarArrayUnlock(result);
  end;

end;

function OleVariantToMemoryStream(OV : OleVariant) : TMemoryStream;
var
  Data : PByteArray;
  Size : integer;
begin
  result := TMemoryStream.Create;
  try
    Size := VarArrayHighBound(OV, 1) - VarArrayLowBound(OV,1) + 1;
    Data := VarArrayLock(OV);
    try
      result.position := 0;
      result.WriteBuffer(Data^,Size);
    finally
      VarArrayUnlock(OV);
    end;
  except
    result.free;
    result := nil;
  end;
end;

function RecordsetToMemoryStream(const ARecordset : _recordset): TMemoryStream;
var
  vStreamAdapter : TStreamAdapter;
begin
  result := TMemoryStream.Create;
  vStreamAdapter := TStreamAdapter.Create(result);
  try
    if not (CoMarshalInterface((vStreamAdapter as IStream),
            IID_Recordset20,
            ARecordset,
            MSHCTX_LOCAL,
            nil,
            MSHLFLAGS_NORMAL) = S_OK) then
    begin
      raise Exception.CreateFmt('Unable to marshal recordset.  %s',
                                [SysErrorMessage(GetLastError)]);
    end;
  finally
    vStreamAdapter.free;
  end;
end;

function MemoryStreamToRecordset(const AMemoryStream : TMemoryStream) : _recordset;
var
  vStreamAdapter : TStreamAdapter;
begin
  result := CoRecordset.Create;
  vStreamAdapter := TStreamAdapter.Create(AMemoryStream);
  try
    if not CoUnmarshalInterface((vStreamAdapter as IStream), IID_Recordset20, result) = S_OK then
    begin
      raise Exception.CreateFmt('Unable to unmarshal recordset.  %s',
                                [SysErrorMessage(GetLastError)]);

    end;
  finally
    vStreamAdapter.free;
  end;

end;

function BundleRecordset( const ARecordset : _recordset ) : olevariant;
begin
  result := MemoryStreamToOlevariant( RecordsetToMemoryStream( ARecordset ) );
end;

function UnbundleRecordset(const OV : Olevariant) : _recordset;
begin
  result := MemoryStreamToRecordset( OleVariantToMemoryStream(OV) );
end;

end.
